#!/usr/bin/env python3
"""
not1mm Contest logger
Email: michael.bridak@gmail.com
GPL V3
Class: LookupService
Purpose: Lookup callsigns with online services.
"""

# pylint: disable=unused-import, c-extension-no-member, no-member, invalid-name, too-many-lines
# pylint: disable=logging-fstring-interpolation, line-too-long, no-name-in-module

import logging
import os
from json import loads

from PyQt6.QtWidgets import QDockWidget
from PyQt6.QtCore import pyqtSignal

import not1mm.fsutils as fsutils

# from not1mm.lib.multicast import Multicast
from not1mm.lib.lookup import QRZlookup, HamQTH

logger = logging.getLogger(__name__)


class LookupService(QDockWidget):
    """The Lookup Service class."""

    message = pyqtSignal(dict)

    def __init__(self):
        super().__init__()
        self._udpwatch = None

        self.look_up = None
        self.settings = self.get_settings()
        if self.settings:

            if self.settings.get("useqrz"):
                self.look_up = QRZlookup(
                    self.settings.get("lookupusername", ""),
                    self.settings.get("lookuppassword", ""),
                )

            if self.settings.get("usehamqth"):
                self.look_up = HamQTH(
                    self.settings.get("lookupusername", ""),
                    self.settings.get("lookuppassword", ""),
                )

    def get_settings(self) -> dict:
        """Get the settings."""
        if os.path.exists(fsutils.CONFIG_FILE):
            with open(fsutils.CONFIG_FILE, "rt", encoding="utf-8") as file_descriptor:
                return loads(file_descriptor.read())
        return {}

    def msg_from_main(self, packet):
        """"""
        if packet.get("cmd", "") == "LOOKUP_CALL":
            if self.look_up:
                call = packet.get("call", "")
                if call:
                    result = self.look_up.lookup(call)
                    cmd = {}
                    cmd["cmd"] = "LOOKUP_RESPONSE"
                    cmd["result"] = result
                    self.message.emit(cmd)
            return

        if packet.get("cmd", "") == "REFRESH_LOOKUP":
            self.settings = self.get_settings()
            self.look_up = None
            if self.settings.get("useqrz"):
                self.look_up = QRZlookup(
                    self.settings.get("lookupusername", ""),
                    self.settings.get("lookuppassword", ""),
                )

            if self.settings.get("usehamqth"):
                self.look_up = HamQTH(
                    self.settings.get("lookupusername", ""),
                    self.settings.get("lookuppassword", ""),
                )



"""SAC CW plugin"""

# pylint: disable=invalid-name, c-extension-no-member, unused-import


import datetime
import logging

# import time
# from not1mm.lib.ham_utility import get_logged_band
from pathlib import Path

from PyQt6 import QtWidgets

from not1mm.lib.plugin_common import gen_adif, imp_adif, get_points, online_score_xml
from not1mm.lib.version import __version__

logger = logging.getLogger(__name__)

EXCHANGE_HINT = "#"

name = "SAC CW"
cabrillo_name = "SAC-CW"
mode = "CW"  # CW SSB BOTH RTTY
# columns = [0, 1, 2, 3, 4, 5, 6, 9, 11, 15]
columns = [
    "YYYY-MM-DD HH:MM:SS",
    "Call",
    "Freq",
    "Snt",
    "Rcv",
    "SentNr",
    "RcvNr",
    "PTS",
]

advance_on_space = [True, True, True, True, True]

# 1 once per contest, 2 work each band, 3 each band/mode, 4 no dupe checking
dupe_type = 2

scandinavian_prefixes = [
    "JW",
    "JX",
    "LA",
    "LB",
    "LC",
    "LG",
    "LI",
    "LJ",
    "LN",
    "OF",
    "OG",
    "OH",
    "OI",
    "OFØ",
    "OGØ",
    "OHØ",
    "OJØ",
    "OX",
    "XP",
    "OW",
    "OY",
    "5P",
    "5Q",
    "OU",
    "OV",
    "OZ",
    "7S",
    "8S",
    "SA",
    "SB",
    "SC",
    "SD",
    "SE",
    "SF",
    "SG",
    "SH",
    "SI",
    "SJ",
    "SK",
    "SL",
    "SM",
    "TF",
]


def init_contest(self):
    """setup plugin"""
    set_tab_next(self)
    set_tab_prev(self)
    interface(self)
    self.next_field = self.other_2


def interface(self):
    """Setup user interface"""
    self.field1.show()
    self.field2.show()
    self.field3.show()
    self.field4.show()
    self.snt_label.setText("SNT")
    self.field1.setAccessibleName("RST Sent")
    self.other_label.setText("SentNR")
    self.field3.setAccessibleName("Sent Number")
    self.exch_label.setText("RcvNR")
    self.field4.setAccessibleName("Received Number")


def reset_label(self):  # pylint: disable=unused-argument
    """reset label after field cleared"""


def set_tab_next(self):
    """Set TAB Advances"""
    self.tab_next = {
        self.callsign: self.sent,
        self.sent: self.receive,
        self.receive: self.other_1,
        self.other_1: self.other_2,
        self.other_2: self.callsign,
    }


def set_tab_prev(self):
    """Set TAB Advances"""
    self.tab_prev = {
        self.callsign: self.other_2,
        self.sent: self.callsign,
        self.receive: self.sent,
        self.other_1: self.receive,
        self.other_2: self.other_1,
    }


def set_contact_vars(self):
    """Contest Specific"""
    self.contact["SNT"] = self.sent.text()
    self.contact["RCV"] = self.receive.text()
    self.contact["SentNr"] = self.other_1.text()
    self.contact["NR"] = self.other_2.text()


def predupe(self):  # pylint: disable=unused-argument
    """called after callsign entered"""


def prefill(self):
    """Fill SentNR"""
    # result = self.database.get_serial()
    # serial_nr = str(result.get("serial_nr", "1")).zfill(3)
    serial_nr = str(self.current_sn).zfill(3)
    if serial_nr == "None":
        serial_nr = "001"
    if len(self.other_1.text()) == 0:
        self.other_1.setText(serial_nr)


def points(self):
    """Calc point"""
    # 7.1 For Scandinavian stations:
    #   EUROPEAN stations, outside Scandinavia, are worth two (2) points for every complete QSO.
    #   NON-EUROPEAN stations are worth three (3) points for every complete QSO.
    # 7.2 For non-Scandinavian stations:
    #   EUROPEAN stations receive one (1) point for every complete Scandinavian QSO.
    #   NON-EUROPEAN stations receive one (1) point for every complete Scandinavian QSO on
    #       14, 21, and 28 MHz and three (3) points for every complete QSO on 3.5 and 7 MHz.

    if self.contact_is_dupe > 0:
        return 0

    myprimary_pfx = ""
    mycontinent = ""
    hisprimary_pfx = ""
    hiscontinent = ""

    result = self.cty_lookup(self.station.get("Call", ""))
    if result:
        item = result.get(next(iter(result)))
        myprimary_pfx = item.get("primary_pfx", "")
        mycontinent = item.get("continent", "")

    result = self.cty_lookup(self.contact.get("Call", ""))
    if result:
        item = result.get(next(iter(result)))
        hisprimary_pfx = item.get("primary_pfx", "")
        hiscontinent = item.get("continent", "")

    if (
        myprimary_pfx in scandinavian_prefixes
        and hisprimary_pfx not in scandinavian_prefixes
    ):
        if hiscontinent == "EU":
            return 2
        return 3
    if (
        myprimary_pfx not in scandinavian_prefixes
        and hisprimary_pfx in scandinavian_prefixes
    ):
        if mycontinent == "EU":
            return 1
        if self.contact.get("Band", 0) in ["3.5", "7"]:
            return 3
        if self.contact.get("Band", 0) in ["14", "21", "28"]:
            return 1

    # Something wrong
    return 0


def show_mults(self):
    """Return display string for mults"""
    myprimary_pfx = ""
    mult_count = 0

    result = self.cty_lookup(self.station.get("Call", ""))
    if result is not None:
        item = result.get(next(iter(result)))
        myprimary_pfx = item.get("primary_pfx", "")

    if myprimary_pfx in scandinavian_prefixes:
        result = self.database.fetch_country_band_count()
        mult_count = result.get("cb_count", 0)
    else:
        query = f"SELECT count(DISTINCT(CountryPrefix || ':' || substr(WPXPrefix,3,1) || ':' || Band)) as mults from DXLOG where ContestNR = {self.pref.get('contest', '1')} AND CountryPrefix IN ('JW', 'JX', 'LA', 'LB', 'LC', 'LG', 'LI', 'LJ' , 'LN', 'OF', 'OG', 'OH', 'OI', 'OFØ', 'OGØ', 'OHØ', 'OJØ', 'OX', 'XP', 'OW', 'OY', '5P', '5Q', 'OU', 'OV', 'OZ', '7S', '8S', 'SA', 'SB', 'SC', 'SD', 'SE', 'SF', 'SG', 'SH', 'SI', 'SJ', 'SK', 'SL', 'SM', 'TF');"
        result = self.database.exec_sql(query)
        mult_count = result.get("mults", 0)
    return mult_count


def show_qso(self):
    """Return qso count"""
    result = self.database.fetch_qso_count()
    if result:
        return int(result.get("qsos", 0))
    return 0


def calc_score(self):
    """Return calculated score"""
    result = self.database.fetch_points()
    if result is not None:
        score = result.get("Points", "0")
        if score is None:
            score = "0"
        contest_points = int(score)
        mults = show_mults(self)
        return contest_points * mults
    return 0


def adif(self):
    """Call the generate ADIF function"""
    gen_adif(self, cabrillo_name, "SAC-CW")


def output_cabrillo_line(line_to_output, ending, file_descriptor, file_encoding):
    """"""
    print(
        line_to_output.encode(file_encoding, errors="ignore").decode(),
        end=ending,
        file=file_descriptor,
    )


def cabrillo(self, file_encoding):
    """Generates Cabrillo file. Maybe."""
    # https://www.cqwpx.com/cabrillo.htm
    logger.debug("******Cabrillo*****")
    logger.debug("Station: %s", f"{self.station}")
    logger.debug("Contest: %s", f"{self.contest_settings}")
    now = datetime.datetime.now()
    date_time = now.strftime("%Y-%m-%d_%H-%M-%S")
    filename = (
        str(Path.home())
        + "/"
        + f"{self.station.get('Call', '').upper()}_{cabrillo_name}_{date_time}.log"
    )
    logger.debug("%s", filename)
    log = self.database.fetch_all_contacts_asc()
    try:
        with open(filename, "w", encoding=file_encoding, newline="") as file_descriptor:
            output_cabrillo_line(
                "START-OF-LOG: 3.0",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CREATED-BY: Not1MM v{__version__}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CONTEST: {cabrillo_name}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.station.get("Club", ""):
                output_cabrillo_line(
                    f"CLUB: {self.station.get('Club', '').upper()}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"CALLSIGN: {self.station.get('Call','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"LOCATION: {self.station.get('ARRLSection', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-OPERATOR: {self.contest_settings.get('OperatorCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-ASSISTED: {self.contest_settings.get('AssistedCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-BAND: {self.contest_settings.get('BandCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            mode = self.contest_settings.get("ModeCategory", "")
            if mode in ["SSB+CW", "SSB+CW+DIGITAL"]:
                mode = "MIXED"
            output_cabrillo_line(
                f"CATEGORY-MODE: {mode}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-TRANSMITTER: {self.contest_settings.get('TransmitterCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.contest_settings.get("OverlayCategory", "") != "N/A":
                output_cabrillo_line(
                    f"CATEGORY-OVERLAY: {self.contest_settings.get('OverlayCategory','')}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"GRID-LOCATOR: {self.station.get('GridSquare','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-POWER: {self.contest_settings.get('PowerCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )

            output_cabrillo_line(
                f"CLAIMED-SCORE: {calc_score(self)}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            ops = ""
            list_of_ops = self.database.get_ops()
            for op in list_of_ops:
                ops += f"{op.get('Operator', '')}, "
            if self.station.get("Call", "") not in ops:
                ops += f"@{self.station.get('Call','')}"
            else:
                ops = ops.rstrip(", ")
            output_cabrillo_line(
                f"OPERATORS: {ops}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"NAME: {self.station.get('Name', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS: {self.station.get('Street1', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-CITY: {self.station.get('City', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-STATE-PROVINCE: {self.station.get('State', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-POSTALCODE: {self.station.get('Zip', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-COUNTRY: {self.station.get('Country', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"EMAIL: {self.station.get('Email', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            for contact in log:
                the_date_and_time = contact.get("TS", "")
                themode = contact.get("Mode", "")
                if themode == "LSB" or themode == "USB":
                    themode = "PH"
                frequency = str(round(contact.get("Freq", "0"))).rjust(5)

                loggeddate = the_date_and_time[:10]
                loggedtime = the_date_and_time[11:13] + the_date_and_time[14:16]
                output_cabrillo_line(
                    f"QSO: {frequency} {themode} {loggeddate} {loggedtime} "
                    f"{contact.get('StationPrefix', '').ljust(13)} "
                    f"{str(contact.get('SNT', '')).ljust(3)} "
                    f"{str(contact.get('SentNr', '')).ljust(6)} "
                    f"{contact.get('Call', '').ljust(13)} "
                    f"{str(contact.get('RCV', '')).ljust(3)} "
                    f"{str(contact.get('NR', '')).ljust(6)}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line("END-OF-LOG:", "\r\n", file_descriptor, file_encoding)
        self.show_message_box(f"Cabrillo saved to: {filename}")
    except IOError as exception:
        logger.critical("cabrillo: IO error: %s, writing to %s", exception, filename)
        self.show_message_box(f"Error saving Cabrillo: {exception} {filename}")
        return


def recalculate_mults(self):
    """Recalculates multipliers after change in logged qso."""


def process_esm(self, new_focused_widget=None, with_enter=False):
    """ESM State Machine"""

    # self.pref["run_state"]

    # -----===== Assigned F-Keys =====-----
    # self.esm_dict["CQ"]
    # self.esm_dict["EXCH"]
    # self.esm_dict["QRZ"]
    # self.esm_dict["AGN"]
    # self.esm_dict["HISCALL"]
    # self.esm_dict["MYCALL"]
    # self.esm_dict["QSOB4"]

    # ----==== text fields ====----
    # self.callsign
    # self.sent
    # self.receive
    # self.other_1
    # self.other_2

    if new_focused_widget is not None:
        self.current_widget = self.inputs_dict.get(new_focused_widget)

    # print(f"checking esm {self.current_widget=} {with_enter=} {self.pref.get("run_state")=}")

    for a_button in [
        self.F1,
        self.F2,
        self.F3,
        self.F4,
        self.F5,
        self.F6,
        self.F7,
        self.F8,
        self.F9,
        self.F10,
        self.F11,
        self.F12,
    ]:
        self.restore_button_color(a_button)

    buttons_to_send = []

    if self.pref.get("run_state"):
        if self.current_widget == "callsign":
            if len(self.callsign.text()) < 3:
                self.make_button_green(self.esm_dict["CQ"])
                buttons_to_send.append(self.esm_dict["CQ"])
            elif len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["HISCALL"])
                self.make_button_green(self.esm_dict["EXCH"])
                buttons_to_send.append(self.esm_dict["HISCALL"])
                buttons_to_send.append(self.esm_dict["EXCH"])

        elif self.current_widget == "other_2":
            if self.other_2.text() == "":
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])
            elif self.other_2.text().isnumeric():
                self.make_button_green(self.esm_dict["QRZ"])
                buttons_to_send.append(self.esm_dict["QRZ"])
                buttons_to_send.append("LOGIT")
            else:
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)
    else:
        if self.current_widget == "callsign":
            if len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["MYCALL"])
                buttons_to_send.append(self.esm_dict["MYCALL"])

        elif self.current_widget == "other_2":
            if self.other_2.text() == "":
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])
            elif self.other_2.text().isnumeric():
                self.make_button_green(self.esm_dict["EXCH"])
                buttons_to_send.append(self.esm_dict["EXCH"])
                buttons_to_send.append("LOGIT")
            else:
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)


def get_mults(self):
    """Get mults for RTC XML"""
    mults = {}
    mults["wpxprefix"] = show_mults(self)
    return mults


def just_points(self):
    """Get points for RTC XML"""
    return get_points(self)


# def add_test_data(self):
#     """"""
#     filename = "/home/mbridak/Nextcloud/dev/not1mm/research/sac/sac_log.txt"
#     try:
#         with open(filename, "rt", encoding="utf-8") as file_descriptor:
#             lines = file_descriptor.readlines()
#             for line in lines:
#                 field = line.split()
#                 print(f"{field[8]} {field[10]}")
#                 self.callsign.setText(str(field[8]))
#                 time.sleep(1.0)
#                 self.other_2.setText(str(field[10]))
#                 time.sleep(1.0)

#     except (FileNotFoundError, IndexError) as err:
#         print(f"{err=}")


# def set_self(the_outie):
#     """..."""
#     globals()["ALTEREGO"] = the_outie


# def ft8_handler(the_packet: dict):
#     """Process FT8 QSO packets

#     FlDigi
#     {
#             'FREQ': '7.029500',
#             'CALL': 'DL2DSL',
#             'MODE': 'RTTY',
#             'NAME': 'BOB',result = ALTEREGO.database.fetch_call_exists(the_packet.get("CALL", ""))
#         if result.get("call_count", ""):
#             ALTEREGO.contact["IsMultiplier1"] = 0
#         else:
#             ALTEREGO.contact["IsMultiplier1"] = 1
#             'QSO_DATE': '20240904',
#             'QSO_DATE_OFF': '20240904',
#             'TIME_OFF': '212825',
#             'TIME_ON': '212800',
#             'RST_RCVD': '599',
#             'RST_SENT': '599',
#             'BAND': '40M',
#             'COUNTRY': 'FED. REP. OF GERMANY',
#             'CQZ': '14',
#             'STX': '000',
#             'STX_STRING': '1D ORG',
#             'CLASS': '1D',
#             'ARRL_SECT': 'DX',
#             'TX_PWR': '0',
#             'OPERATOR': 'K6GTE',
#             'STATION_CALLSIGN': 'K6GTE',
#             'MY_GRIDSQUARE': 'DM13AT',
#             'MY_CITY': 'ANAHEIM, CA',
#             'MY_STATE': 'CA'
#         }

#     """
#     logger.debug(f"{the_packet=}")
#     if ALTEREGO is not None:
#         ALTEREGO.callsign.setText(the_packet.get("CALL"))
#         ALTEREGO.contact["Call"] = the_packet.get("CALL", "")
#         ALTEREGO.contact["SNT"] = ALTEREGO.sent.text()
#         ALTEREGO.contact["RCV"] = ALTEREGO.receive.text()
#         ALTEREGO.contact["NR"] = the_packet.get("SRX_STRING", "001")
#         ALTEREGO.other_2.setText(the_packet.get("SRX_STRING", "001"))
#         ALTEREGO.other_1.setText(the_packet.get("STX_STRING", "001"))
#         ALTEREGO.contact["SentNr"] = the_packet.get("STX_STRING", "001")

#         ALTEREGO.contact["Mode"] = the_packet.get("MODE", "ERR")
#         ALTEREGO.contact["Freq"] = round(float(the_packet.get("FREQ", "0.0")) * 1000, 2)

#         ALTEREGO.contact["QSXFreq"] = round(
#             float(the_packet.get("FREQ", "0.0")) * 1000, 2
#         )
#         ALTEREGO.contact["Band"] = get_logged_band(
#             str(int(float(the_packet.get("FREQ", "0.0")) * 1000000))
#         )
#         result = ALTEREGO.cty_lookup(the_packet.get("CALL"))
#         if result:
#             for a in result.items():
#                 entity = a[1].get("entity", "")
#                 cq = a[1].get("cq", "")
#                 itu = a[1].get("itu", "")
#                 continent = a[1].get("continent")
#                 primary_pfx = a[1].get("primary_pfx", "")
#                 ALTEREGO.contact["CountryPrefix"] = primary_pfx
#                 ALTEREGO.contact["Continent"] = continent
#         ALTEREGO.radio_state["vfoa"] = int(
#             float(the_packet.get("FREQ", "0.0")) * 1000000
#         )
#         logger.debug(f"{ALTEREGO.contact=}")
#         ALTEREGO.save_contact()



"""ARI International DX Contest"""

# pylint: disable=invalid-name, c-extension-no-member, unused-import, line-too-long
# pyright: ignore[reportUndefinedVariable]
# pylance: disable=reportUndefinedVariable
# ruff: noqa: F821
# ruff: noqa: F401

# Status:	            Active
# Geographic Focus:     Worldwide
# Participation:        Worldwide
# Awards:	            Worldwide
# Mode:	                Phone, CW, RTTY
# Bands:	            80, 40, 20, 15, 10m
# Classes:	            Single Op (CW/SSB/RTTY/Mixed)(Low/High)
#                       Single Op Overlays (Italian only): (Rookie/Youth)
#                       Multi-Single
#                       Multi-Multi
#                       SWL
# Max power:            HP: >100 watts
#                       LP: 100 watts
# Exchange:             I: RS(T) + 2-letter province
#                       non-I: RS(T) + Serial No.
# Work stations:        Once per mode per band
# QSO Points:           0 points per QSO with same country
#                       1 point per QSO with different country same continent
#                       3 points per QSO with different continent
#                       10 points per QSO with I/IS0/IT9 stations
# Multipliers:          Each Italian province once per band
#                       Each DXCC country once per band
# Score Calculation:    Total score = total QSO points x total mults
# E-mail logs to:       (none)
# Upload log at:        https://www.ari.it/contest-hf/ari-international/log-upload.html
# Mail logs to:         (none)
# Find rules at:        https://www.ari.it/
# Cabrillo name:	    ARI-DX


import datetime
import logging
import platform

from pathlib import Path

from PyQt6 import QtWidgets

from not1mm.lib.ham_utility import get_logged_band
from not1mm.lib.plugin_common import gen_adif, imp_adif
from not1mm.lib.version import __version__

logger = logging.getLogger(__name__)

EXCHANGE_HINT = "Prov or '#'"

name = "ARI International DX"
cabrillo_name = "ARI-DX"
mode = "BOTH"  # CW SSB BOTH RTTY

columns = [
    "YYYY-MM-DD HH:MM:SS",
    "Call",
    "Freq",
    "Mode",
    "Snt",
    "Rcv",
    "SentNr",
    "RcvNr",
    "PTS",
]

advance_on_space = [True, True, True, True, True]

# 1 once per contest, 2 work each band, 3 each band/mode, 4 no dupe checking
dupe_type = 3


def init_contest(self):
    """setup plugin"""
    set_tab_next(self)
    set_tab_prev(self)
    interface(self)
    self.next_field = self.other_2


def interface(self):
    """Setup user interface"""
    self.field1.show()
    self.field2.show()
    self.field3.show()
    self.field4.show()
    self.snt_label.setText("SNT")
    self.field1.setAccessibleName("RST Sent")
    self.exch_label.setText("Prov or SN")
    self.field4.setAccessibleName("Province or Serial Number")


def reset_label(self):  # pylint: disable=unused-argument
    """reset label after field cleared"""


def set_tab_next(self):
    """Set TAB Advances"""
    self.tab_next = {
        self.callsign: self.sent,
        self.sent: self.receive,
        self.receive: self.other_1,
        self.other_1: self.other_2,
        self.other_2: self.callsign,
    }


def set_tab_prev(self):
    """Set TAB Advances"""
    self.tab_prev = {
        self.callsign: self.other_2,
        self.sent: self.callsign,
        self.receive: self.sent,
        self.other_1: self.receive,
        self.other_2: self.other_1,
    }


def set_contact_vars(self):
    """Contest Specific"""
    self.contact["SNT"] = self.sent.text()
    self.contact["RCV"] = self.receive.text()
    self.contact["NR"] = self.other_2.text().upper()
    self.contact["SentNr"] = self.other_1.text()


def predupe(self):  # pylint: disable=unused-argument
    """called after callsign entered"""


def prefill(self):
    """Fill sentnr"""
    # result = self.database.get_serial()
    # serial_nr = str(result.get("serial_nr", "1")).zfill(3)
    serial_nr = str(self.current_sn).zfill(3)
    if serial_nr == "None":
        serial_nr = "001"

    exchange = self.contest_settings.get("SentExchange", "").replace("#", serial_nr)
    if len(self.other_1.text()) == 0:
        self.other_1.setText(exchange)


def points(self) -> int:
    """Calc point"""

    # QSO Points:           0 points per QSO with same country
    #                       1 point per QSO with different country same continent
    #                       3 points per QSO with different continent
    #                       10 points per QSO with I/IS0/IT9 stations

    if self.contact_is_dupe > 0:
        return 0

    result = self.cty_lookup(self.station.get("Call", ""))
    if result is not None:
        item = result.get(next(iter(result)))
        mycountry = item.get("primary_pfx", "")
        # myentity = item.get("entity", "")
        mycontinent = item.get("continent", "")

    result = self.cty_lookup(self.contact.get("Call", ""))
    print(f"{result=}")
    if result is not None:
        item = result.get(next(iter(result)))
        hiscountry = item.get("primary_pfx", "")
        # hisentity = item.get("entity", "")
        hiscontinent = item.get("continent", "")

    _points = 0

    if mycountry == hiscountry:
        _points = 0
    if mycountry != hiscountry and mycontinent == hiscontinent:
        _points = 1
    if mycontinent != hiscontinent:
        _points = 3
    if hiscountry in ("I", "IS0", "IT9"):
        _points = 10

    return _points


def show_mults(self):
    """Return display string for mults"""

    # Multipliers:          Each Italian province once per band
    #                       Each DXCC country once per band

    _country = 0
    _province = 0

    sql = (
        "select count(DISTINCT(NR || ':' || Band)) as mult_count from dxlog "
        f"where ContestNR = {self.database.current_contest} and typeof(NR) = 'text';"
    )
    result = self.database.exec_sql(sql)
    if result:
        _province = result.get("mult_count", 0)

    sql = (
        "select count(DISTINCT(CountryPrefix || ':' || Band)) as cb_count from dxlog "
        f"where ContestNR = {self.database.current_contest} and CountryPrefix NOT in ('I', 'IS');"
    )
    result2 = self.database.exec_sql(sql)
    if result2:
        _country = int(result2.get("cb_count", 0))

    return _country + _province


def show_qso(self):
    """Return qso count"""
    result = self.database.fetch_qso_count()
    if result:
        return int(result.get("qsos", 0))
    return 0


def calc_score(self):
    """Return calculated score"""
    result = self.database.fetch_points()
    if result is not None:
        score = result.get("Points", "0")
        if score is None:
            score = "0"
        contest_points = int(score)
        mults = int(show_mults(self))
        return contest_points * mults
    return 0


def adif(self):
    """Call the generate ADIF function"""
    gen_adif(self, cabrillo_name, cabrillo_name)


def output_cabrillo_line(line_to_output, ending, file_descriptor, file_encoding):
    """"""
    print(
        line_to_output.encode(file_encoding, errors="ignore").decode(),
        end=ending,
        file=file_descriptor,
    )


def cabrillo(self, file_encoding):
    """Generates Cabrillo file. Maybe."""
    # https://www.cw160.com/cabrillo.htm
    logger.debug("******Cabrillo*****")
    logger.debug("Station: %s", f"{self.station}")
    logger.debug("Contest: %s", f"{self.contest_settings}")
    now = datetime.datetime.now()
    date_time = now.strftime("%Y-%m-%d_%H-%M-%S")
    filename = (
        str(Path.home())
        + "/"
        + f"{self.station.get('Call', '').upper()}_{cabrillo_name}_{date_time}.log"
    )
    logger.debug("%s", filename)
    log = self.database.fetch_all_contacts_asc()
    try:
        with open(filename, "w", encoding=file_encoding, newline="") as file_descriptor:
            output_cabrillo_line(
                "START-OF-LOG: 3.0",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CREATED-BY: Not1MM v{__version__}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CONTEST: {cabrillo_name}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.station.get("Club", ""):
                output_cabrillo_line(
                    f"CLUB: {self.station.get('Club', '').upper()}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"CALLSIGN: {self.station.get('Call','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"LOCATION: {self.station.get('ARRLSection', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-OPERATOR: {self.contest_settings.get('OperatorCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-ASSISTED: {self.contest_settings.get('AssistedCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-BAND: {self.contest_settings.get('BandCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            mode = self.contest_settings.get("ModeCategory", "")
            if mode in ["SSB+CW", "SSB+CW+DIGITAL"]:
                mode = "MIXED"
            output_cabrillo_line(
                f"CATEGORY-MODE: {mode}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-TRANSMITTER: {self.contest_settings.get('TransmitterCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.contest_settings.get("OverlayCategory", "") != "N/A":
                output_cabrillo_line(
                    f"CATEGORY-OVERLAY: {self.contest_settings.get('OverlayCategory','')}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"GRID-LOCATOR: {self.station.get('GridSquare','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-POWER: {self.contest_settings.get('PowerCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )

            output_cabrillo_line(
                f"CLAIMED-SCORE: {calc_score(self)}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            ops = ""
            list_of_ops = self.database.get_ops()
            for op in list_of_ops:
                ops += f"{op.get('Operator', '')}, "
            if self.station.get("Call", "") not in ops:
                ops += f"@{self.station.get('Call','')}"
            else:
                ops = ops.rstrip(", ")
            output_cabrillo_line(
                f"OPERATORS: {ops}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"NAME: {self.station.get('Name', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS: {self.station.get('Street1', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-CITY: {self.station.get('City', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-STATE-PROVINCE: {self.station.get('State', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-POSTALCODE: {self.station.get('Zip', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-COUNTRY: {self.station.get('Country', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"EMAIL: {self.station.get('Email', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            for contact in log:
                the_date_and_time = contact.get("TS", "")
                themode = contact.get("Mode", "")
                if themode == "LSB" or themode == "USB":
                    themode = "PH"
                if themode == "RTTY":
                    themode = "RY"
                frequency = str(round(contact.get("Freq", "0"))).rjust(5)

                loggeddate = the_date_and_time[:10]
                loggedtime = the_date_and_time[11:13] + the_date_and_time[14:16]
                output_cabrillo_line(
                    f"QSO: {frequency} {themode} {loggeddate} {loggedtime} "
                    f"{contact.get('StationPrefix', '').ljust(13)} "
                    f"{str(contact.get('SNT', '')).ljust(3)} "
                    f"{str(contact.get('SentNr', '')).upper().ljust(6)} "
                    f"{contact.get('Call', '').ljust(13)} "
                    f"{str(contact.get('RCV', '')).ljust(3)} "
                    f"{str(contact.get('NR', '')).upper().ljust(6)}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line("END-OF-LOG:", "\r\n", file_descriptor, file_encoding)
        self.show_message_box(f"Cabrillo saved to: {filename}")
    except IOError as exception:
        logger.critical("cabrillo: IO error: %s, writing to %s", exception, filename)
        self.show_message_box(f"Error saving Cabrillo: {exception} {filename}")
        return


def trigger_update(self):
    """Triggers the log window to update."""
    cmd = {}
    cmd["cmd"] = "UPDATELOG"
    if self.log_window:
        self.log_window.msg_from_main(cmd)


def recalculate_mults(self):
    """Recalculates multipliers after change in logged qso."""


def process_esm(self, new_focused_widget=None, with_enter=False):
    """ESM State Machine"""

    # self.pref["run_state"]

    # -----===== Assigned F-Keys =====-----
    # self.esm_dict["CQ"]
    # self.esm_dict["EXCH"]
    # self.esm_dict["QRZ"]
    # self.esm_dict["AGN"]
    # self.esm_dict["HISCALL"]
    # self.esm_dict["MYCALL"]
    # self.esm_dict["QSOB4"]

    # ----==== text fields ====----
    # self.callsign
    # self.sent
    # self.receive
    # self.other_1
    # self.other_2

    if new_focused_widget is not None:
        self.current_widget = self.inputs_dict.get(new_focused_widget)

    # print(f"checking esm {self.current_widget=} {with_enter=} {self.pref.get("run_state")=}")

    for a_button in [
        self.esm_dict["CQ"],
        self.esm_dict["EXCH"],
        self.esm_dict["QRZ"],
        self.esm_dict["AGN"],
        self.esm_dict["HISCALL"],
        self.esm_dict["MYCALL"],
        self.esm_dict["QSOB4"],
    ]:
        if a_button is not None:
            self.restore_button_color(a_button)

    buttons_to_send = []

    if self.pref.get("run_state"):
        if self.current_widget == "callsign":
            if len(self.callsign.text()) < 3:
                self.make_button_green(self.esm_dict["CQ"])
                buttons_to_send.append(self.esm_dict["CQ"])
            elif len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["HISCALL"])
                self.make_button_green(self.esm_dict["EXCH"])
                buttons_to_send.append(self.esm_dict["HISCALL"])
                buttons_to_send.append(self.esm_dict["EXCH"])

        elif self.current_widget in ["other_2"]:
            if self.other_2.text() == "":
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])
            else:
                self.make_button_green(self.esm_dict["QRZ"])
                buttons_to_send.append(self.esm_dict["QRZ"])
                buttons_to_send.append("LOGIT")

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)
    else:
        if self.current_widget == "callsign":
            if len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["MYCALL"])
                buttons_to_send.append(self.esm_dict["MYCALL"])

        elif self.current_widget in ["other_2"]:
            if self.other_2.text() == "":
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])
            else:
                self.make_button_green(self.esm_dict["EXCH"])
                buttons_to_send.append(self.esm_dict["EXCH"])
                buttons_to_send.append("LOGIT")

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)


def populate_history_info_line(self):
    result = self.database.fetch_call_history(self.callsign.text())
    if result:
        self.history_info.setText(f"{result.get('Call', '')}, {result.get('Sect', '')}")
    else:
        self.history_info.setText("")


def check_call_history(self):
    """"""
    result = self.database.fetch_call_history(self.callsign.text())
    if result:
        self.history_info.setText(f"{result.get('UserText','')}")
        if self.other_2.text() == "":
            self.other_2.setText(f"{result.get('Sect', '')}")


def set_self(the_outie):
    """..."""
    globals()["ALTEREGO"] = the_outie


def ft8_handler(the_packet: dict):
    print(f"{the_packet=}")
    """Process FT8 QSO packets
    FT8
    {
        'CALL': 'KE0OG',
        'GRIDSQUARE': 'DM10AT',
        'MODE': 'FT8',
        'RST_SENT': '',
        'RST_RCVD': '',
        'QSO_DATE': '20210329',
        'TIME_ON': '183213',
        'QSO_DATE_OFF': '20210329',
        'TIME_OFF': '183213',
        'BAND': '20M',
        'FREQ': '14.074754',
        'STATION_CALLSIGN': 'K6GTE',
        'MY_GRIDSQUARE': 'DM13AT',
        'CONTEST_ID': 'ARRL-FIELD-DAY',
        'SRX_STRING': '1D UT',
        'CLASS': '1D',
        'ARRL_SECT': 'UT'
    }
    FlDigi
    {
        'CALL': 'K5TUS', 
        'MODE': 'RTTY', 
        'FREQ': '14.068415', 
        'BAND': '20M', 
        'QSO_DATE': '20250103', 
        'TIME_ON': '2359', 
        'QSO_DATE_OFF': '20250103', 
        'TIME_OFF': '2359', 
        'NAME': '', 
        'QTH': '', 
        'STATE': 'ORG', 
        'VE_PROV': '', 
        'COUNTRY': 'USA', 
        'RST_SENT': '599', 
        'RST_RCVD': '599', 
        'TX_PWR': '0', 
        'CNTY': '', 
        'DXCC': '', 
        'CQZ': '5', 
        'IOTA': '', 
        'CONT': '', 
        'ITUZ': '', 
        'GRIDSQUARE': '', 
        'QSLRDATE': '', 
        'QSLSDATE': '', 
        'EQSLRDATE': '', 
        'EQSLSDATE': '', 
        'LOTWRDATE': '', 
        'LOTWSDATE': '', 
        'QSL_VIA': '', 
        'NOTES': '', 
        'SRX': '', 
        'STX': '000', 
        'SRX_STRING': '', 
        'STX_STRING': 'CA', 


        'SRX': '666', 
        'STX': '000', 
        'SRX_STRING': '', 
        'STX_STRING': 'CA',

        'SRX': '004', 'STX': '000', 'SRX_STRING': '', 'STX_STRING': '#',

        'CLASS': '', 
        'ARRL_SECT': '', 
        'OPERATOR': 'K6GTE', 
        'STATION_CALLSIGN': 'K6GTE', 
        'MY_GRIDSQUARE': 'DM13AT', 
        'MY_CITY': 'ANAHEIM, CA', 
        'CHECK': '', 
        'AGE': '', 
        'TEN_TEN': '', 
        'CWSS_PREC': '', 
        'CWSS_SECTION': '', 
        'CWSS_SERNO': '', 
        'CWSS_CHK': ''
    }

    """
    logger.debug(f"{the_packet=}")
    if ALTEREGO is not None:  # type: ignore
        ALTEREGO.callsign.setText(the_packet.get("CALL"))  # type: ignore
        ALTEREGO.contact["Call"] = the_packet.get("CALL", "")  # type: ignore
        ALTEREGO.contact["SNT"] = the_packet.get("RST_SENT", "599")  # type: ignore
        ALTEREGO.contact["RCV"] = the_packet.get("RST_RCVD", "599")  # type: ignore

        sent_string = the_packet.get("STX_STRING", "")
        if sent_string != "":
            ALTEREGO.contact["SentNr"] = sent_string  # type: ignore
            ALTEREGO.other_1.setText(str(sent_string))  # type: ignore
        else:
            ALTEREGO.contact["SentNr"] = the_packet.get("STX", "000")  # type: ignore
            ALTEREGO.other_1.setText(str(the_packet.get("STX", "000")))  # type: ignore

        rx_string = the_packet.get("STATE", "")
        if rx_string != "":
            ALTEREGO.contact["NR"] = rx_string  # type: ignore
            ALTEREGO.other_2.setText(str(rx_string))  # type: ignore
        else:
            ALTEREGO.contact["NR"] = the_packet.get("SRX", "000")  # type: ignore
            ALTEREGO.other_2.setText(str(the_packet.get("SRX", "000")))  # type: ignore

        ALTEREGO.contact["Mode"] = the_packet.get("MODE", "ERR")  # type: ignore
        ALTEREGO.contact["Freq"] = round(float(the_packet.get("FREQ", "0.0")) * 1000, 2)  # type: ignore
        ALTEREGO.contact["QSXFreq"] = round(  # type: ignore
            float(the_packet.get("FREQ", "0.0")) * 1000, 2
        )
        ALTEREGO.contact["Band"] = get_logged_band(  # type: ignore
            str(int(float(the_packet.get("FREQ", "0.0")) * 1000000))
        )
        logger.debug(f"{ALTEREGO.contact=}")  # type: ignore

        ALTEREGO.save_contact()  # type: ignore



from PyQt6.QtWidgets import QDockWidget
from PyQt6.QtGui import QBrush, QColor

# from PyQt6.QtCore import Qt
from PyQt6.QtCore import pyqtSignal, Qt
from PyQt6 import uic, QtWidgets
import not1mm.fsutils as fsutils
from not1mm.lib.database import DataBase
import os
from json import loads
from json.decoder import JSONDecodeError

import logging

logger = logging.getLogger(__name__)


class DXCCWindow(QDockWidget):
    message = pyqtSignal(dict)
    ctyfile = {}
    dbname = None
    db = None
    model = None
    pref = {}
    columns = {
        0: "DXCC",
        1: "160m",
        2: "80m",
        3: "40m",
        4: "20m",
        5: "15m",
        6: "10m",
        7: "Total",
    }

    def __init__(self, action):
        super().__init__()
        self.action = action
        self.active = False
        uic.loadUi(fsutils.APP_DATA_PATH / "dxcc_tracker.ui", self)
        self.dxcc_table.setColumnCount(len(self.columns))
        for column_number, column_name in self.columns.items():
            self.dxcc_table.setHorizontalHeaderItem(
                column_number, QtWidgets.QTableWidgetItem(column_name)
            )
        self.setWindowTitle("DXCC Tracker")
        self.load_pref()

        self.dbname = fsutils.USER_DATA_PATH / self.pref.get(
            "current_database", "ham.db"
        )
        self.database = DataBase(self.dbname, fsutils.USER_DATA_PATH)

        self.database.current_contest = self.pref.get("contest", 0)

        self.get_log()
        self.dxcc_table.resizeColumnsToContents()
        self.dxcc_table.resizeRowsToContents()
        self.load_cty_data()

    def setActive(self, mode: bool):
        self.active = bool(mode)

    def load_cty_data(self):
        try:
            with open(
                fsutils.APP_DATA_PATH / "cty.json", "rt", encoding="utf-8"
            ) as c_file:
                self.ctyfile = loads(c_file.read())
        except (IOError, JSONDecodeError, TypeError):
            logging.critical("There was an error parsing the BigCity file.")

    def dxcc_lookup(self, dxcc: str) -> str:
        """Lookup callsign in cty.dat file.

        Parameters
        ----------
        callsign : str
        callsign to lookup

        Returns
        -------
        return : dict
        {'entity': 'European Russia', 'cq': 16, 'itu': 29, 'continent': 'EU', 'lat': 53.65, 'long': -41.37, 'tz': 4.0, 'len': 2, 'primary_pfx': 'UA', 'exact_match': False}
        """

        result = self.ctyfile.get(dxcc, {})
        return result.get("entity", "")

    def get_log(self):
        """dxcc_table"""

        # result=[
        # {'CountryPrefix': 'K', '160m': 0, '80m': 0, '40m': 0, '20m': 7, '15m': 0, '10m': 0, 'Total': 7},
        # {'CountryPrefix': 'XE', '160m': 0, '80m': 0, '40m': 0, '20m': 1, '15m': 0, '10m': 0, 'Total': 1},
        # {'CountryPrefix': 'G', '160m': 0, '80m': 0, '40m': 0, '20m': 1, '15m': 0, '10m': 0, 'Total': 1}
        # ]
        if not self.active:
            return
        result = self.database.fetch_dxcc_by_band_count()
        self.dxcc_table.setRowCount(0)
        for row_number, row_data in enumerate(result):
            self.dxcc_table.insertRow(row_number)
            for column, data in enumerate(row_data.values()):
                item = QtWidgets.QTableWidgetItem(str(data))
                if column == 0:
                    item.setToolTip(self.dxcc_lookup(data))
                if column > 0 and column < 7:
                    if data == 0:
                        item.setBackground(QBrush(QColor(44, 138, 44)))
                    else:
                        item.setBackground(QBrush(QColor(155, 100, 100)))
                self.dxcc_table.setItem(row_number, column, item)
        self.dxcc_table.resizeColumnsToContents()
        self.dxcc_table.resizeRowsToContents()

    def scrollToDXCC(self, item: str):
        """Scrolls dxcc table to the item"""
        if (
            self.active is True
            and self.isVisible()
            and isinstance(item, str)
            and self.scrolltoCheckBox.isChecked()
        ):
            matchingitems = self.dxcc_table.findItems(
                item.upper(), Qt.MatchFlag.MatchExactly
            )
            if matchingitems:
                matcheditem = matchingitems[0]  # take the first
                self.dxcc_table.scrollToItem(matcheditem)

    def load_pref(self) -> None:
        """
        Loads the preferences from the config file into the self.pref dictionary.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        try:
            if os.path.exists(fsutils.CONFIG_FILE):
                with open(
                    fsutils.CONFIG_FILE, "rt", encoding="utf-8"
                ) as file_descriptor:
                    self.pref = loads(file_descriptor.read())
                    logger.info("%s", self.pref)
            else:
                self.pref["current_database"] = "ham.db"

        except IOError as exception:
            logger.critical("Error: %s", exception)

    def load_new_db(self) -> None:
        """
        If the database changes reload it.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        self.load_pref()
        self.dbname = fsutils.USER_DATA_PATH / self.pref.get(
            "current_database", "ham.db"
        )
        self.database = DataBase(self.dbname, fsutils.APP_DATA_PATH)
        self.database.current_contest = self.pref.get("contest", 0)
        self.contact = self.database.empty_contact
        self.get_log()

    def msg_from_main(self, msg):
        """"""

        if self.active is True and self.isVisible():
            if msg.get("cmd", "") in (
                "UPDATELOG",
                "CONTACTCHANGED",
                "DELETE",
                "DELETED",
            ):
                ...
                self.get_log()
            if msg.get("cmd", "") == "NEWDB":
                ...
                self.load_new_db()
                self.get_log()
            if msg.get("cmd", "") == "SCROLLTODXCC":
                self.scrollToDXCC(msg.get("dxcc", ""))

    def closeEvent(self, event) -> None:
        self.action.setChecked(False)



"""WAG - Worked All Germany plugin"""

# pylint: disable=invalid-name, unused-argument, unused-variable, c-extension-no-member, unused-import

#   Worked All Germany Contest
#   Status:	Active
#   Geographic Focus: Germany
#   Participation:	Worldwide
#   Awards:	Worldwide
#   Mode: CW, SSB
#   Bands: 80m, 40m, 20m, 15m, 10m
#   Classes:
#       Single operator, CW, low power
#       Single operator, CW, high power
#       Single operator, SSB, low power (new since 2019)
#       Single operator, SSB, high power (new since 2019)
#       Single operator, mixed, low power
#       Single operator, mixed, high power
#       Single operator, mixed, QRP
#       Multi operator
#       Trainee (Auszubildende): Participiants with trainee callsign. (Prefixes DN1-DN8 and /T stations.)
#   Max power: HP: >100W, LP < 100W, QRP < 5W
#   Exchange: RST + DOK
#       non-Member (DL): RST + 'NM' or 'no member'
#       outside DL: RST + serial number
#   Work stations: Once per band and mode
#   QSO Points:	1 point per QSO
#   Multipliers:
#       DL: DOK (DARC/VFDB), 'NM' does not count + each DXCC/WAE per band
#   Score Calculation:	Total score = total QSO points x total mults
#   Mail logs to: (none)
#   Find rules at:	https://www.darc.de/der-club/referate/conteste/wag-contest/regeln/
#   Upload logs at: https://dxhf2.darc.de/~waglog/upload.cgi?form=referat&lang=de
#   Cabrillo name: WAG


import datetime
import logging
import re

from pathlib import Path

from not1mm.lib.plugin_common import (
    gen_adif,
)  # , imp_adif, get_points, online_score_xml
from not1mm.lib.version import __version__

logger = logging.getLogger(__name__)

__plugin_version__ = "0.1"

EXCHANGE_HINT = "DOK/NM or #"

name = "WAG"
contest_id = "WAG"
cabrillo_name = "WAG"
mode = "BOTH"  # CW SSB BOTH RTTY
columns = [
    "YYYY-MM-DD HH:MM:SS",
    "Call",
    "Freq",
    "Snt",
    "Rcv",
    "SentNr",
    "RcvNr",
    "M1",
    "PTS",
]

advance_on_space = [True, True, True, True, True]

# 1 once per contest, 2 work each band, 3 each band/mode, 4 no dupe checking
dupe_type = 3


def init_contest(self):
    """setup plugin"""
    set_tab_next(self)
    set_tab_prev(self)
    interface(self)
    self.next_field = self.other_2

    # Is the contester located in DL?
    result = self.cty_lookup(self.station.get("Call", ""))
    if result is not None:
        item = result.get(next(iter(result)))
        if item.get("primary_pfx", "") == "DL":
            self.is_german = True
        else:
            self.is_german = False
    logger.debug(f"Contest station is DL: {self.is_german}")


def interface(self):
    """Setup user interface"""
    self.field1.show()
    self.field2.show()
    self.field3.show()
    self.field4.show()
    self.snt_label.setText("SNT")
    self.other_label.setText("DOK/# Sent")
    self.field3.setAccessibleName("RST Sent")
    self.exch_label.setText("DOK/# RCVD")
    self.field4.setAccessibleName("DOK/NM or Number")


def reset_label(self):
    """reset label after field cleared"""

    # Reset the dupe_indicator to his correct text as we used it for warnings.
    self.dupe_indicator.setText("Dupe!")


def set_tab_next(self):
    """Set TAB Advances"""
    self.tab_next = {
        self.callsign: self.sent,
        self.sent: self.receive,
        self.receive: self.other_1,
        self.other_1: self.other_2,
        self.other_2: self.callsign,
    }


def set_tab_prev(self):
    """Set TAB Advances"""
    self.tab_prev = {
        self.callsign: self.other_2,
        self.sent: self.callsign,
        self.receive: self.sent,
        self.other_1: self.receive,
        self.other_2: self.other_1,
    }


def set_contact_vars(self):
    """Set contact variables and calculate multis"""

    self.contact["SNT"] = self.sent.text()
    self.contact["RCV"] = self.receive.text()
    self.contact["NR"] = self.other_2.text()
    self.contact["SentNr"] = self.other_1.text()

    dok = self.contact["NR"].upper()
    dxcc = self.contact.get("CountryPrefix", "")
    band = self.contact.get("Band", "")
    district = get_district(dok)

    # Multiplier
    # non-DL worked DL
    if dxcc == "DL" and not isinstance(dok, int) and dok != "NM" and not self.is_german:
        query = (
            f"select count(*) as dok_count from dxlog where 1=1 "
            f"and NR like '{district}%' "
            f"and Band = '{band}' "
            f"and ContestNR = {self.pref.get('contest', '1')};"
        )
        logger.debug(query)
        result = self.database.exec_sql(query)
        logger.debug(result)
        count = int(result.get("dok_count", 0))
        if count == 0:
            self.contact["IsMultiplier1"] = 1
            logger.debug(f"{self.contact.get('all')} is a Multi")
        else:
            self.contact["IsMultiplier1"] = 0
            logger.debug(f"{self.contact.get('Call')} is not a Multi")

    # Multiplier
    # DL worked any station
    if self.is_german:
        query = (
            f"select count(*) as dxcc_count from dxlog where 1=1 "
            f"and CountryPrefix = '{dxcc}' "
            f"and Band = '{band}' "
            f"and ContestNR = {self.pref.get('contest', '1')};"
        )
        logger.debug(query)
        result = self.database.exec_sql(query)
        logger.debug(result)
        count = int(result.get("dxcc_count", 0))
        if count == 0:
            self.contact["IsMultiplier1"] = 1
            logger.debug(f"{self.contact.get('Call')} is a Multi")
        else:
            self.contact["IsMultiplier1"] = 0
            logger.debug(f"{self.contact.get('Call')} is not a Multi")


def predupe(self):
    """prefill his exchange with last known values"""

    # Check if the typed in callsign should be worked (DL only in WAG for non-DL stations!)
    dxcc = self.contact.get("CountryPrefix", "")
    if not self.is_german and dxcc != "DL":
        logger.debug("DL only in WAG!")
        self.dupe_indicator.setText("DL stations only!")
        self.dupe_indicator.show()


def prefill(self):
    """Fill SentNR"""
    sent_sxchange_setting = self.contest_settings.get("SentExchange", "")
    if sent_sxchange_setting.strip() == "#":
        result = self.database.get_serial()
        serial_nr = str(result.get("serial_nr", "1")).zfill(3)
        if serial_nr == "None":
            serial_nr = "001"
        if len(self.other_1.text()) == 0:
            self.other_1.setText(serial_nr)
    else:
        self.other_1.setText(sent_sxchange_setting)

    if self.other_2.text() == "":
        call = self.callsign.text().upper()
        query = f"select NR from dxlog where Call = '{call}' and ContestName = '{contest_id}' order by ts desc;"
        logger.debug(query)
        result = self.database.exec_sql(query)
        logger.debug("%s", f"{result}")
        if result:
            if isinstance(result.get("NR", ""), str):
                self.other_2.setText(str(result.get("NR", "")))


def points(self):
    """Calculate points"""

    # Dupe
    if self.contact_is_dupe > 0:
        return 0

    # non-DL worked DL
    # non-DL worked non-DL
    # DL worked DL
    # DL worked EU
    # DL worked DX
    if not self.is_german and self.contact.get("CountryPrefix", "") == "DL":
        return 3
    elif not self.is_german and self.contact.get("CountryPrefix", "") != "DL":
        return 0
    elif self.is_german and self.contact.get("CountryPrefix", "") == "DL":
        return 1
    elif self.is_german and self.contact.get("Continent", "") == "EU":
        return 3
    elif self.is_german and self.contact.get("Continent", "") != "EU":
        return 5
    else:
        return 0


def show_mults(self):
    """Return number of multis"""
    result = self.database.fetch_mult_count(1)
    if result:
        return int(result.get("count", 0))
    return 0


def show_qso(self):
    """Return qso count"""
    result = self.database.fetch_qso_count()
    if result:
        return int(result.get("qsos", 0))
    return 0


def calc_score(self):
    """Return calculated score"""
    result = self.database.fetch_points()
    if result is not None:
        score = result.get("Points", "0")
        if score is None:
            score = "0"
        contest_points = int(score)
        mults = show_mults(self)
        return contest_points * mults
    return 0


def adif(self):
    """Call the generate ADIF function"""
    gen_adif(self, cabrillo_name, contest_id)


def output_cabrillo_line(line_to_output, ending, file_descriptor, file_encoding):
    """"""
    print(
        line_to_output.encode(file_encoding, errors="ignore").decode(),
        end=ending,
        file=file_descriptor,
    )


def cabrillo(self, file_encoding):
    """Generates Cabrillo file. Maybe."""
    # https://www.cqwpx.com/cabrillo.htm
    logger.debug("******Cabrillo*****")
    logger.debug("Station: %s", f"{self.station}")
    logger.debug("Contest: %s", f"{self.contest_settings}")
    now = datetime.datetime.now()
    date_time = now.strftime("%Y-%m-%d_%H-%M-%S")
    filename = (
        str(Path.home())
        + "/"
        + f"{self.station.get('Call', '').upper()}_{cabrillo_name}_{date_time}.log"
    )
    logger.debug("%s", filename)
    log = self.database.fetch_all_contacts_asc()
    try:
        with open(filename, "w", encoding=file_encoding, newline="") as file_descriptor:
            output_cabrillo_line(
                "START-OF-LOG: 3.0",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CREATED-BY: Not1MM v{__version__} - WAG-Contest-Plugin v{__plugin_version__} by DL6CQ",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CONTEST: {cabrillo_name}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.station.get("Club", ""):
                output_cabrillo_line(
                    f"CLUB: {self.station.get('Club', '').upper()}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"CALLSIGN: {self.station.get('Call','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"LOCATION: {self.station.get('ARRLSection', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-OPERATOR: {self.contest_settings.get('OperatorCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-ASSISTED: {self.contest_settings.get('AssistedCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-BAND: {self.contest_settings.get('BandCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            mode = self.contest_settings.get("ModeCategory", "")
            if mode in ["SSB+CW", "SSB+CW+DIGITAL"]:
                mode = "MIXED"
            output_cabrillo_line(
                f"CATEGORY-MODE: {mode}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-TRANSMITTER: {self.contest_settings.get('TransmitterCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            if self.contest_settings.get("OverlayCategory", "") != "N/A":
                output_cabrillo_line(
                    f"CATEGORY-OVERLAY: {self.contest_settings.get('OverlayCategory','')}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line(
                f"GRID-LOCATOR: {self.station.get('GridSquare','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"CATEGORY-POWER: {self.contest_settings.get('PowerCategory','')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )

            output_cabrillo_line(
                f"CLAIMED-SCORE: {calc_score(self)}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            ops = ""
            list_of_ops = self.database.get_ops()
            for op in list_of_ops:
                ops += f"{op.get('Operator', '')}, "
            if self.station.get("Call", "") not in ops:
                ops += f"@{self.station.get('Call','')}"
            else:
                ops = ops.rstrip(", ")
            output_cabrillo_line(
                f"OPERATORS: {ops}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"NAME: {self.station.get('Name', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS: {self.station.get('Street1', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-CITY: {self.station.get('City', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-STATE-PROVINCE: {self.station.get('State', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-POSTALCODE: {self.station.get('Zip', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"ADDRESS-COUNTRY: {self.station.get('Country', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            output_cabrillo_line(
                f"EMAIL: {self.station.get('Email', '')}",
                "\r\n",
                file_descriptor,
                file_encoding,
            )
            for contact in log:
                the_date_and_time = contact.get("TS", "")
                themode = contact.get("Mode", "")
                if themode == "LSB" or themode == "USB":
                    themode = "PH"
                frequency = str(round(contact.get("Freq", "0"))).rjust(5)

                loggeddate = the_date_and_time[:10]
                loggedtime = the_date_and_time[11:13] + the_date_and_time[14:16]
                output_cabrillo_line(
                    f"QSO: {frequency} {themode} {loggeddate} {loggedtime} "
                    f"{contact.get('StationPrefix', '').ljust(13)} "
                    f"{str(contact.get('SNT', '')).ljust(3)} "
                    f"{str(contact.get('SentNr', '')).ljust(6)} "
                    f"{contact.get('Call', '').ljust(13)} "
                    f"{str(contact.get('RCV', '')).ljust(3)} "
                    f"{str(contact.get('NR', '')).ljust(6)}",
                    "\r\n",
                    file_descriptor,
                    file_encoding,
                )
            output_cabrillo_line("END-OF-LOG:", "\r\n", file_descriptor, file_encoding)
        self.show_message_box(f"Cabrillo saved to: {filename}")
    except IOError as exception:
        logger.critical("cabrillo: IO error: %s, writing to %s", exception, filename)
        self.show_message_box(f"Error saving Cabrillo: {exception} {filename}")
        return


def recalculate_mults(self):
    """Recalculates multipliers after change in logged qso."""

    all_contacts = self.database.fetch_all_contacts_asc()
    if not all_contacts:
        return

    for contact in all_contacts:

        contact["IsMultiplier1"] = 0

        time_stamp = contact.get("TS", "")
        dok = contact.get("NR", "")
        dxcc = contact.get("CountryPrefix", "")
        band = contact.get("Band", "")
        district = get_district(str(dok))

        # Multiplier
        # non-DL worked DL
        if (
            dxcc == "DL"
            and not isinstance(dok, int)
            and dok != "NM"
            and not self.is_german
        ):
            query = (
                f"select count(*) as dok_count from dxlog where 1=1 "
                f"and TS < '{time_stamp}' "
                f"and NR like '{district}%' "
                f"and Band = '{band}' "
                f"and ContestNR = {self.pref.get('contest', '1')};"
            )
            logger.debug(query)
            result = self.database.exec_sql(query)
            logger.debug(result)
            count = int(result.get("dok_count", 0))
            if count == 0:
                contact["IsMultiplier1"] = 1
                logger.debug(f"{contact.get('Call')} is a Multi")
            else:
                contact["IsMultiplier1"] = 0
                logger.debug(f"{contact.get('Call')} is not a Multi")

        # Multiplier
        # DL worked any station
        if self.is_german:
            query = (
                f"select count(*) as dxcc_count from dxlog where 1=1 "
                f"and TS < '{time_stamp}' "
                f"and CountryPrefix = '{dxcc}' "
                f"and Band = '{band}' "
                f"and ContestNR = {self.pref.get('contest', '1')};"
            )
            logger.debug(query)
            result = self.database.exec_sql(query)
            count = int(result.get("dxcc_count", 0))

            if count == 0:
                contact["IsMultiplier1"] = 1
            else:
                contact["IsMultiplier1"] = 0

        self.database.change_contact(contact)
    trigger_update(self)


def process_esm(self, new_focused_widget=None, with_enter=False):
    """ESM State Machine"""

    # self.pref["run_state"]

    # -----===== Assigned F-Keys =====-----
    # self.esm_dict["CQ"]
    # self.esm_dict["EXCH"]
    # self.esm_dict["QRZ"]
    # self.esm_dict["AGN"]
    # self.esm_dict["HISCALL"]
    # self.esm_dict["MYCALL"]
    # self.esm_dict["QSOB4"]

    # ----==== text fields ====----
    # self.callsign
    # self.sent
    # self.receive
    # self.other_1
    # self.other_2

    if new_focused_widget is not None:
        self.current_widget = self.inputs_dict.get(new_focused_widget)

    # print(f"checking esm {self.current_widget=} {with_enter=} {self.pref.get('run_state')=}")

    for a_button in [
        self.F1,
        self.F2,
        self.F3,
        self.F4,
        self.F5,
        self.F6,
        self.F7,
        self.F8,
        self.F9,
        self.F10,
        self.F11,
        self.F12,
    ]:
        self.restore_button_color(a_button)

    buttons_to_send = []

    if self.pref.get("run_state"):
        if self.current_widget == "callsign":
            if len(self.callsign.text()) < 3:
                self.make_button_green(self.esm_dict["CQ"])
                buttons_to_send.append(self.esm_dict["CQ"])
            elif len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["HISCALL"])
                self.make_button_green(self.esm_dict["EXCH"])
                buttons_to_send.append(self.esm_dict["HISCALL"])
                buttons_to_send.append(self.esm_dict["EXCH"])

        elif self.current_widget in ["other_1", "other_2"]:
            if self.other_2.text() == "" or self.other_1.text() == "":
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])
            else:
                self.make_button_green(self.esm_dict["QRZ"])
                buttons_to_send.append(self.esm_dict["QRZ"])
                buttons_to_send.append("LOGIT")

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)
    else:
        if self.current_widget == "callsign":
            if len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["MYCALL"])
                buttons_to_send.append(self.esm_dict["MYCALL"])

        elif self.current_widget in ["other_1", "other_2"]:
            if self.other_2.text() == "" or self.other_1.text() == "":
                self.make_button_green(self.esm_dict["AGN"])
                buttons_to_send.append(self.esm_dict["AGN"])
            else:
                self.make_button_green(self.esm_dict["EXCH"])
                buttons_to_send.append(self.esm_dict["EXCH"])
                buttons_to_send.append("LOGIT")

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)


def populate_history_info_line(self):
    """Show history info if available"""
    result = self.database.fetch_call_history(self.callsign.text())
    if result:
        self.history_info.setText(
            f"{result.get('Call', '')}, {result.get('Exch1', '')}"
        )
    else:
        self.history_info.setText("")


def check_call_history(self):
    """"""
    result = self.database.fetch_call_history(self.callsign.text())
    if result:
        self.history_info.setText(f"{result.get('UserText','')}")
        if self.other_2.text() == "":
            # Strip leading whitespace for the case that historyfile is not
            # well formatted, f.e. CA0LL, DOK instead of CA0LL,DOK
            self.other_2.setText(f"{result.get('Exch1', '').lstrip()}")


# def get_mults(self):
#     """Get mults for RTC XML"""
#     mults = {}
#     mults["state"], mults["wpxprefix"] = show_mults(self, rtc=True)
#     return mults


def trigger_update(self):
    """Triggers the log window to update."""
    cmd = {}
    cmd["cmd"] = "UPDATELOG"
    if self.log_window:
        self.log_window.msg_from_main(cmd)


def get_district(dok):
    """
    Get first letter of received DOK (Distrikt)
    With a SDOK (special DOK) we need the first letter (not character!) of the
    SDOK as multi.

    :param dok: a DOK or special DOK
    :type  dok: string
    :return: district - one letter for the DARC district
    :rtype: string
    """

    district = re.search(r"[A-Za-z]", dok)
    if district:
        return district.group(0)
    return None



import sys

from PyQt6.QtWidgets import QApplication, QGridLayout, QPushButton, QStyle, QWidget


class Window(QWidget):
    def __init__(self):
        super().__init__()

        icons = sorted(
            [attr for attr in dir(QStyle.StandardPixmap) if attr.startswith("SP_")]
        )
        layout = QGridLayout()

        for n, name in enumerate(icons):
            btn = QPushButton(name)

            pixmapi = getattr(QStyle.StandardPixmap, name)
            icon = self.style().standardIcon(pixmapi)
            btn.setIcon(icon)
            layout.addWidget(btn, int(n / 4), int(n % 4))

        self.setLayout(layout)


app = QApplication(sys.argv)

w = Window()
w.show()

app.exec()



import datetime
import logging
import os

# import sys

from PyQt6 import uic, QtGui
from PyQt6.QtWidgets import QDockWidget
from PyQt6.QtCore import pyqtSignal

import not1mm.fsutils as fsutils
from json import loads
from json.decoder import JSONDecodeError

logger = logging.getLogger(__name__)

# chat_history  QTextBrowser
# chat_input    QLineedit


class ChatWindow(QDockWidget):
    """The stats window. Shows something important."""

    message = pyqtSignal(dict)
    mycall = ""
    poll_time = datetime.datetime.now() + datetime.timedelta(milliseconds=1000)

    def __init__(self, action):
        super().__init__()
        self.action = action
        self.active: bool = False
        # self.load_pref()
        uic.loadUi(fsutils.APP_DATA_PATH / "chat.ui", self)
        self.chat_input.returnPressed.connect(self.send_chat)

    def send_chat(self):
        """Sends UDP chat packet with text entered in chat_entry field."""
        chat_string = self.chat_input.text()
        packet = {"cmd": "CHAT"}
        #  packet["sender"] = self.preference.get("mycall", "")
        packet["message"] = chat_string
        self.message.emit(packet)
        self.chat_input.setText("")

    def display_chat(self, sender, body):
        """Displays the chat history."""

        if self.mycall in body.upper():
            self.chat_history.setTextColor(QtGui.QColor(245, 121, 0))
        self.chat_history.insertPlainText(f"\n{sender}: {body}")
        self.chat_history.setTextColor(QtGui.QColor(211, 215, 207))
        self.chat_history.ensureCursorVisible()

    def msg_from_main(self, packet):
        """"""

        if packet.get("cmd", "") == "CHAT":
            # {"cmd": "CHAT", "sender": "N2CQR", "message": "I worked your mama on 80 meters."}
            self.display_chat(packet.get("sender", ""), packet.get("message", ""))
            return
        if packet.get("cmd", "") == "CONTESTSTATUS":
            self.mycall = packet.get("operator", "").upper()
            return

    def setActive(self, mode: bool) -> None:
        self.active: bool = bool(mode)

    def load_pref(self) -> None:
        """
        Load preference file to get current db filename and sets the initial darkmode state.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        try:
            if os.path.exists(fsutils.CONFIG_FILE):
                with open(
                    fsutils.CONFIG_FILE, "rt", encoding="utf-8"
                ) as file_descriptor:
                    self.pref = loads(file_descriptor.read())
                    logger.info(f"loaded config file from {fsutils.CONFIG_FILE}")
            else:
                self.pref["current_database"] = "ham.db"

        except (IOError, JSONDecodeError) as exception:
            logger.critical("Error: %s", exception)

    def closeEvent(self, event) -> None:
        self.action.setChecked(False)


if __name__ == "__main__":
    print("This is not a program.\nTry Again.")



"""Common function(s) for all contest plugins"""

import logging
import datetime
import re
import uuid
import adif_io

from decimal import Decimal
from pathlib import Path

from PyQt6.QtWidgets import QApplication, QDialog, QPushButton, QProgressDialog
from PyQt6.QtCore import QCoreApplication, Qt

from not1mm.lib.ham_utility import get_adif_band, get_not1mm_band, get_not1mm_band_xlog
from not1mm.lib.version import __version__

logger = logging.getLogger(__name__)


def online_score_xml(self):
    """generate online xml"""

    mults = self.contest.get_mults(self)
    the_mults = ""
    for thing in mults:
        the_mults += (
            f'<mult band="total" mode="ALL" type="{thing}">{mults.get(thing,0)}</mult>'
        )

    the_points = self.contest.just_points(self)

    the_date_time = datetime.datetime.now(datetime.timezone.utc).isoformat(" ")[:19]
    assisted = self.contest_settings.get("AssistedCategory", "")
    bands = self.contest_settings.get("BandCategory", "")
    modes = self.contest_settings.get("ModeCategory", "")
    if modes in ["SSB+CW", "SSB+CW+DIGITAL"]:
        modes = "MIXED"
    xmiter = self.contest_settings.get("TransmitterCategory", "")
    ops = self.contest_settings.get("OperatorCategory", "")
    overlay = self.contest_settings.get("OverlayCategory", "")
    power = self.contest_settings.get("PowerCategory", "")

    the_xml = (
        '<?xml version="1.0"?>'
        "<dynamicresults>"
        f"<contest>{self.contest.cabrillo_name}</contest>"
        f'<call>{self.station.get("Call", "")}</call>'
        # <ops>NR9Q</ops>
        f'<class power="{power}" assisted = "{assisted}" transmitter="{xmiter}" ops="{ops}" bands="{bands}" mode="{modes}" overlay="{overlay}"></class>'
        f"<club>{self.station.get('Club', '')}</club>"
        "<soft>Not1MM</soft>"
        f"<version>{__version__}</version>"
        "<qth>"
        # <dxcccountry>K</dxcccountry>
        f"<cqzone>{self.station.get('CQZone','')}</cqzone>"
        f"<iaruzone>{self.station.get('IARUZone','')}</iaruzone>"
        f"<arrlsection>{self.station.get('ARRLSection', '')}</arrlsection>"
        f"<stprvoth>{self.station.get('State','')}</stprvoth>"
        f"<grid6>{self.station.get('GridSquare','')}</grid6>"
        "</qth>"
        "<breakdown>"
        f'<qso band="total" mode="ALL">{self.contest.show_qso(self)}</qso>'
        f"{the_mults}"
        f'<point band="total" mode="ALL">{the_points}</point>'
        "</breakdown>"
        f"<score>{self.contest.calc_score(self)}</score>"
        f"<timestamp>{the_date_time}</timestamp>"
        "</dynamicresults>"
    )
    return the_xml


def get_points(self):
    """Return raw points before mults"""
    result = self.database.fetch_points()
    if result:
        if result.get("Points", 0) is not None:
            return int(result.get("Points", 0))
    return 0


def gen_adif(self, cabrillo_name: str, contest_id=""):
    """
    Creates an ADIF file of the contacts made.
    """
    now = datetime.datetime.now()
    date_time = now.strftime("%Y-%m-%d_%H-%M-%S")
    station_callsign = self.station.get("Call", "").upper()
    filename = (
        str(Path.home()) + "/" + f"{station_callsign}_{cabrillo_name}_{date_time}.adi"
    )
    log = self.database.fetch_all_contacts_asc()
    try:
        with open(filename, "w", encoding="utf-8", newline="") as file_descriptor:
            print("Not1MM ADIF export", end="\r\n", file=file_descriptor)
            print("<ADIF_VER:5>3.1.5", end="\r\n", file=file_descriptor)
            print("<EOH>", end="\r\n", file=file_descriptor)
            for contact in log:
                hiscall = contact.get("Call", "")
                hisname = contact.get("Name", "")
                the_date_and_time = contact.get("TS", "")
                themode = contact.get("Mode", "")
                if themode in ("CW", "CW-U", "CW-L", "CW-R", "CWR"):
                    themode = "CW"
                if cabrillo_name in ("CQ-WW-RTTY", "WEEKLY-RTTY"):
                    themode = "RTTY"
                frequency = str(Decimal(str(contact.get("Freq", 0))) / 1000)
                band = get_adif_band(Decimal(str(contact.get("Freq", 0))) / 1000)
                sentrst = contact.get("SNT", "")
                rcvrst = contact.get("RCV", "")
                sentnr = str(contact.get("SentNr", "0"))
                rcvnr = str(contact.get("NR", "0"))
                grid = contact.get("GridSquare", "")
                pfx = contact.get("CountryPrefix", "")
                comment = contact.get("Comment", "")
                loggeddate = the_date_and_time[:10]
                loggedtime = (
                    the_date_and_time[11:13]
                    + the_date_and_time[14:16]
                    + the_date_and_time[17:20]
                )
                print(
                    f"<QSO_DATE:{len(''.join(loggeddate.split('-')))}:d>"
                    f"{''.join(loggeddate.split('-'))}",
                    end="\r\n",
                    file=file_descriptor,
                )

                try:
                    print(
                        f"<TIME_ON:{len(loggedtime)}>{loggedtime}",
                        end="\r\n",
                        file=file_descriptor,
                    )
                except TypeError:
                    ...

                try:
                    print(
                        f"<STATION_CALLSIGN:{len(station_callsign)}>{station_callsign}",
                        end="\r\n",
                        file=file_descriptor,
                    )
                except TypeError:
                    ...

                try:
                    print(
                        f"<CALL:{len(hiscall)}>{hiscall.upper()}",
                        end="\r\n",
                        file=file_descriptor,
                    )
                except TypeError:
                    ...

                try:
                    if len(hisname):
                        print(
                            f"<NAME:{len(hisname)}>{hisname.title()}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                try:
                    if themode in ("USB", "LSB"):
                        print(
                            f"<MODE:3>SSB\r\n<SUBMODE:{len(themode)}>{themode}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                    else:
                        print(
                            f"<MODE:{len(themode)}>{themode}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                try:
                    print(
                        f"<BAND:{len(band)}>{band}",
                        end="\r\n",
                        file=file_descriptor,
                    )
                except TypeError:
                    ...

                try:
                    print(
                        f"<FREQ:{len(frequency)}>{frequency}",
                        end="\r\n",
                        file=file_descriptor,
                    )
                except TypeError:
                    ...

                try:
                    print(
                        f"<RST_SENT:{len(sentrst)}>{sentrst}",
                        end="\r\n",
                        file=file_descriptor,
                    )
                except TypeError:
                    ...

                try:
                    print(
                        f"<RST_RCVD:{len(rcvrst)}>{rcvrst}",
                        end="\r\n",
                        file=file_descriptor,
                    )
                except TypeError:
                    ...

                try:
                    if cabrillo_name in ("WFD", "ARRL-FD", "ARRL-FIELD-DAY"):
                        sent = self.contest_settings.get("SentExchange", "")
                        if sent:
                            print(
                                f"<STX_STRING:{len(sent)}>{sent.upper()}",
                                end="\r\n",
                                file=file_descriptor,
                            )
                    elif cabrillo_name in ("ICWC-MST"):
                        sent = (
                            f'{self.contest_settings.get("SentExchange", "")} {sentnr}'
                        )
                        if sent:
                            print(
                                f"<STX_STRING:{len(sent)}>{sent.upper()}",
                                end="\r\n",
                                file=file_descriptor,
                            )
                    elif sentnr != "0":
                        print(
                            f"<STX_STRING:{len(sentnr)}>{sentnr}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                # SRX STRING, Contest dependent
                try:
                    # ----------Medium Speed Test------------
                    if cabrillo_name in ("ICWC-MST"):
                        rcv = f"{hisname.upper()} {contact.get('NR', '')}"
                        if len(rcv) > 1:
                            print(
                                f"<SRX_STRING:{len(rcv)}>{rcv.upper()}",
                                end="\r\n",
                                file=file_descriptor,
                            )
                    # ----------Field Days------------
                    elif cabrillo_name in ("WFD", "ARRL-FD", "ARRL-FIELD-DAY"):
                        rcv = (
                            f"{contact.get('Exchange1', '')} {contact.get('Sect', '')}"
                        )
                        if len(rcv) > 1:
                            print(
                                f"<SRX_STRING:{len(rcv)}>{rcv.upper()}",
                                end="\r\n",
                                file=file_descriptor,
                            )
                    # ------------CQ 160---------------
                    elif cabrillo_name in ("CQ-160-CW", "CQ-160-SSB", "WEEKLY-RTTY"):
                        rcv = f"{contact.get('Exchange1', '')}"
                        if len(rcv) > 1:
                            print(
                                f"<SRX_STRING:{len(rcv)}>{rcv.upper()}",
                                end="\r\n",
                                file=file_descriptor,
                            )
                    # --------------K1USN-SST-----------
                    elif cabrillo_name == "K1USN-SST":
                        rcv = f"{contact.get('Name', '')} {contact.get('Sect', '')}"
                        if len(rcv) > 1:
                            print(
                                f"<SRX_STRING:{len(rcv)}>{rcv.upper()}",
                                end="\r\n",
                                file=file_descriptor,
                            )
                    # ------------CQ-WW-DX-RTTY---------
                    elif cabrillo_name == "CQ-WW-RTTY":
                        rcv = f"{str(contact.get('ZN', '')).zfill(2)} {contact.get('Exchange1', 'DX')}"
                        if len(rcv) > 1:
                            print(
                                f"<SRX_STRING:{len(rcv)}>{rcv.upper()}",
                                end="\r\n",
                                file=file_descriptor,
                            )
                    elif rcvnr != "0":
                        print(
                            f"<SRX_STRING:{len(rcvnr)}>{rcvnr}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                try:
                    result = re.match(
                        "[A-R][A-R]([0-9][0-9][A-X][A-X])*([0-9][0-9])?",
                        grid,
                        re.IGNORECASE,
                    )
                    grid = ""
                    if result:
                        grid = result.group()

                    if len(grid[:8]) > 1:
                        print(
                            f"<GRIDSQUARE:{len(grid[:8])}>{grid[:8]}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                try:
                    if len(pfx) > 0:
                        print(
                            f"<PFX:{len(pfx)}>{pfx}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                try:
                    if len(contest_id) > 1:
                        print(
                            f"<CONTEST_ID:{len(contest_id)}>{contest_id}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                try:
                    if len(comment):
                        print(
                            f"<COMMENT:{len(comment)}>{comment}",
                            end="\r\n",
                            file=file_descriptor,
                        )
                except TypeError:
                    ...

                print("<EOR>", end="\r\n", file=file_descriptor)
                print("", end="\r\n", file=file_descriptor)
            self.show_message_box(f"ADIF saved to: {filename}")
    except IOError as error:
        self.show_message_box(f"Error saving ADIF file: {error}")


def imp_adif(self):
    """
    Imports an ADIF file into the current contest.
    """

    filename = self.filepicker("other")
    if not filename:
        return
    logger.debug(f"Selected file '{filename}' to import from")

    # read in content in binary mode (in case of illegal characters)
    try:
        with open(filename, "rb") as file:
            file_content = file.read()
    except Exception as e:
        self.show_message_box(f"Error: {e}")
        return

    # filter out anything beyond 7-bit ASCII
    ascii_content = str("")
    for b in file_content:
        if b < 128:
            ascii_content = ascii_content + chr(b)

    qsos_raw, adif_header = adif_io.read_from_string(ascii_content)
    qsos_sorted = sorted(qsos_raw, key=adif_io.time_on)

    num_qsos = len(qsos_sorted)
    logger.debug(f"Found {num_qsos} QSOs to import")
    self.show_message_box(f"Found {num_qsos} QSOs in\n'{filename}'.")
    if num_qsos == 0:
        return

    self.progress_dialog = QProgressDialog("Validating...", "Cancel", 0, num_qsos, self)
    self.progress_dialog.setWindowModality(Qt.WindowModality.WindowModal)
    # using .show() breaks modality - just start updating

    q_num = 0
    contacts = []
    dupes = 0

    for q in qsos_sorted:
        QCoreApplication.processEvents()
        if self.progress_dialog.wasCanceled():
            self.progress_dialog.close()
            self.show_message_box("Cancelling import.")
            return

        this_contact = self.database.get_empty()

        try:
            temp = adif_io.time_on(q)
            this_contact["TS"] = temp.strftime("%Y-%m-%d %H:%M:%S")
        except KeyError:
            logger.debug("Date/time not found in QSO #{q_num+1}")
            self.progress_dialog.close()
            self.show_message_box(
                f"Date/time not found in QSO #{q_num+1}.\nImport cancelled."
            )
            return

        if q.get("CALL"):
            this_contact["Call"] = q.get("CALL")
        else:
            logger.debug("Callsign not found in QSO #{q_num+1}")
            self.progress_dialog.close()
            self.show_message_box(
                f"Callsign not found in QSO #{q_num+1}.\nImport cancelled."
            )
            return

        # ADIF freq is in MHz, not1mm is in kHz
        if q.get("FREQ"):
            this_contact["Freq"] = float(q.get("FREQ")) * 1000.0
        else:
            logger.debug("Frequency not found in QSO #{q_num+1}")
            self.progress_dialog.close()
            self.show_msgbox(
                f"Valid Frequency not found in QSO #{q_num+1}.\nImport cancelled."
            )
            return

        if q.get("QSXFREQ"):
            this_contact["QSXFreq"] = float(q.get("QSXFREQ")) * 1000.0
        else:
            this_contact["QSXFreq"] = 0.0

        if q.get("MODE"):
            this_contact["Mode"] = q.get("MODE")
        elif q.get("SUBMODE"):
            this_contact["Mode"] = q.get("SUBMODE")
        else:
            logger.debug("Mode not found in QSO #{q_num+1}")
            self.progress_dialog.close()
            self.show_message_box(
                f"Valid Mode not found in QSO #{q_num+1}.\nImport cancelled."
            )
            return

        this_contact["ContestName"] = self.contest.name

        if q.get("SNT"):
            this_contact["SNT"] = q.get("SNT")
        elif q.get("RST_SENT"):
            this_contact["SNT"] = q.get("RST_SENT")

        if q.get("RCV"):
            this_contact["RCV"] = q.get("RCV")
        elif q.get("RST_RCVD"):
            this_contact["RCV"] = q.get("RST_RCVD")

        if q.get("COUNTRYPREFIX"):
            this_contact["CountryPrefix"] = q.get("COUNTRYPREFIX")
        elif q.get("PFX"):
            this_contact["CountryPrefix"] = q.get("PFX")

        if q.get("STATIONPREFIX"):
            this_contact["StationPrefix"] = q.get("STATIONPREFIX")

        if q.get("QTH"):
            this_contact["QTH"] = q.get("QTH")

        if q.get("NAME"):
            this_contact["Name"] = q.get("NAME")

        if q.get("COMMENT"):
            this_contact["Comment"] = q.get("COMMENT")

        if q.get("NR"):
            this_contact["NR"] = q.get("NR")

        if q.get("SECT"):
            this_contact["Sect"] = q.get("SECT")
        elif q.get("ARRL_SECT"):
            this_contact["Sect"] = q.get("ARRL_SECT")

        QCoreApplication.processEvents()

        if q.get("PREC"):
            this_contact["Prec"] = q.get("PREC")

        if q.get("CK"):
            this_contact["CK"] = q.get("CK")

        if q.get("ZN"):
            this_contact["ZN"] = q.get("ZN")
        elif q.get("CQZ"):
            this_contact["ZN"] = q.get("CQZ")

        if q.get("SENTNR"):
            this_contact["SentNr"] = q.get("SENTNR")

        if q.get("POINTS"):
            this_contact["Points"] = q.get("POINTS")
        elif q.get("APP_N1MM_POINTS"):
            this_contact["Points"] = q.get("APP_N1MM_POINTS")

        if q.get("APP_N1MM_MULT1"):
            this_contact["IsMultiplier1"] = q.get("APP_N1MM_MULT1")

        if q.get("APP_N1MM_MULT2"):
            this_contact["IsMultiplier2"] = q.get("APP_N1MM_MULT2")

        if q.get("POWER"):
            this_contact["Power"] = q.get("POWER")
        elif q.get("TX_PWR"):
            this_contact["Power"] = q.get("TX_PWR")

        # ADIF Band is in Meters (eg, "20m"), not1mm is in (float) MHz
        # xlog does not export a Band field, so Band should not be mandatory
        # 1st attempt: ADIF style like "18m"
        temp = str(q.get("BAND"))
        temp = get_not1mm_band(temp.lower())
        # 2nd attempt: no Band field, so take a Freq like "18.160" and double-convert
        temp2 = get_adif_band(float(q.get("FREQ")))  # returns like "18m"
        temp3 = get_not1mm_band(temp2.lower())  # returns like "18.068"
        # 3rd attempt: abbreviated Freq like "18" (ie, from xlog)
        temp4 = get_not1mm_band_xlog(q.get("FREQ"))
        if temp != 0.0:
            this_contact["Band"] = temp
        elif temp3 != 0.0:
            this_contact["Band"] = temp3
        elif temp4 != 0.0:
            this_contact["Band"] = temp4
        else:
            # Well, we tried.
            this_contact["Band"] = 0.0

        if q.get("WPXPREFIX"):
            this_contact["WPXPrefix"] = q.get("WPXPREFIX")

        if q.get("EXCHANGE1"):
            this_contact["Exchange1"] = q.get("EXCHANGE1")
        elif q.get("CLASS"):
            this_contact["Exchange1"] = q.get("CLASS")
        elif q.get("APP_N1MM_EXCHANGE1"):
            this_contact["Exchange1"] = q.get("APP_N1MM_EXCHANGE1")

        if q.get("RADIONR"):
            this_contact["RadioNR"] = q.get("RADIONR")
        elif q.get("APP_N1MM_RADIONR"):
            this_contact["RadioNR"] = q.get("APP_N1MM_RADIONR")
        else:
            this_contact["RadioNR"] = 1

        this_contact["ContestNR"] = self.pref.get("contest", "0")

        if q.get("ISMULTIPLIER3"):
            this_contact["isMultiplier3"] = q.get("ISMULTIPLIER3")
        elif q.get("APP_N1MM_MULT3"):
            this_contact["isMultiplier3"] = q.get("APP_N1MM_MULT3")

        if q.get("MISCTEXT"):
            this_contact["MiscText"] = q.get("MISCTEXT")

        if q.get("ISRUNQSO"):
            this_contact["IsRunQSO"] = q.get("ISRUNQSO")

        if q.get("CONTACTTYPE"):
            this_contact["ContactType"] = q.get("CONTACTTYPE")

        QCoreApplication.processEvents()

        if q.get("RUN1RUN2"):
            this_contact["Run1Run2"] = q.get("RUN1RUN2")
        elif q.get("APP_N1MM_RUN1RUN2"):
            this_contact["Run1Run2"] = q.get("APP_N1MM_RUN1RUN2")
        else:
            this_contact["Run1Run2"] = 1

        if q.get("GRIDSQUARE"):
            this_contact["GridSquare"] = q.get("GRIDSQUARE")

        if q.get("OPERATOR"):
            this_contact["Operator"] = q.get("OPERATOR")
        elif q.get("STATION_CALLSIGN"):
            this_contact["Operator"] = q.get("STATION_CALLSIGN")

        if q.get("CONTINENT"):
            this_contact["Continent"] = q.get("CONTINENT")
        elif q.get("APP_N1MM_CONTINENT"):
            this_contact["Continent"] = q.get("APP_N1MM_CONTINENT")

        if q.get("ROVERLOCATION"):
            this_contact["RoverLocation"] = q.get("ROVERLOCATION")

        if q.get("RADIOINTERFACED"):
            this_contact["RadioInterfaced"] = q.get("RADIOINTERFACED")
        elif q.get("APP_N1MM_RADIOINTERFACED"):
            this_contact["RadioInterfaced"] = q.get("APP_N1MM_RADIOINTERFACED")

        if q.get("NETWORKEDCOMPNR"):
            this_contact["NetworkedCompNr"] = q.get("NETWORKEDCOMPNR")

        if q.get("NETBIOSNAME"):
            this_contact["NetBiosName"] = q.get("NETBIOSNAME")
        elif q.get("APP_N1MM_NETBIOSNAME"):
            this_contact["NetBiosName"] = q.get("APP_N1MM_NETBIOSNAME")
        elif q.get("N3FJP_COMPUTERNAME"):
            this_contact["NetBiosName"] = q.get("N3FJP_COMPUTERNAME")

        if q.get("ISORIGINAL"):
            this_contact["IsOriginal"] = q.get("ISORIGINAL")
        elif q.get("APP_N1MM_ISORIGINAL"):
            this_contact["IsOriginal"] = q.get("APP_N1MM_ISORIGINAL")

        this_contact["ID"] = uuid.uuid4().hex

        if q.get("CLAIMEDQSO"):
            this_contact["CLAIMEDQSO"] = q.get("CLAIMEDQSO")
        elif q.get("APP_N1MM_CLAIMEDQSO"):
            this_contact["CLAIMEDQSO"] = q.get("APP_N1MM_CLAIMEDQSO")

        # is this record a dupe?
        theTS = this_contact["TS"]
        thecall = this_contact["Call"]
        temp = self.database.exec_sql(
            f"select count(*) as isdupe from dxlog where TS = '{theTS}' and call = '{thecall}'"
        )
        if temp["isdupe"] > 0:
            dupes = dupes + 1
        else:
            contacts.append(this_contact.copy())

        this_contact.clear()
        q_num = q_num + 1
        self.progress_dialog.setValue(q_num)
        QCoreApplication.processEvents()

    # All ADIF records have now been mapped.
    # setting to max value forces progress_dialog to close
    self.progress_dialog.setValue(num_qsos)

    logger.debug(f"Found {dupes} duplicate records")
    if dupes > 0:
        self.show_message_box(
            f"NOTE: Found {dupes} duplicate records, which will not be saved."
        )

    # open new progress_dialog for Save progress.
    self.progress_dialog = QProgressDialog(
        "Saving...", "Cancel", 0, len(contacts), self
    )
    self.progress_dialog.setWindowModality(Qt.WindowModality.WindowModal)

    saves = 0
    for my_contact in contacts:
        QCoreApplication.processEvents()
        if self.progress_dialog.wasCanceled():
            self.progress_dialog.close()
            self.show_message_box("Cancelling import in progress.")
            return

        self.database.log_contact(my_contact)

        saves = saves + 1
        self.progress_dialog.setValue(saves)
        QCoreApplication.processEvents()

    self.progress_dialog.setValue(len(contacts))  # forces close
    # update everything
    self.log_window.get_log()

    if self.actionStatistics.isChecked():
        self.statistics_window.get_run_and_total_qs()

    score = self.contest.calc_score(self)
    self.score.setText(str(score))

    mults = self.contest.show_mults(self)
    qsos = self.contest.show_qso(self)
    multstring = f"{qsos}/{mults}"
    self.mults.setText(multstring)

    logger.debug(f"Saved {saves} ADIF records to contest {self.contest.name}")
    self.show_message_box(f"Saved {saves} ADIF records to this contest.")
    return
